/*****************************************************************************/
// Includes

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef WIN32
#include <windows.h>
#else
#include <time.h>
#endif

#include "useful/useful.h"
#include "trajectory/trajectory.h"
#include "dynamics.h"
#include "controller.h"

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/************************************************************************/
// Used for timing how long simulation takes.
// Comment out if necessary, not essential.

#ifdef WIN32
double get_time()
{
  return (double) timeGetTime()/1000.0;
}
#else
double get_time()
{
  struct timespec the_tp;

  clock_gettime( CLOCK_MONOTONIC, &the_tp );
  return ((double) (the_tp.tv_sec)) + 1.0e-9*the_tp.tv_nsec;
}
#endif

/*****************************************************************************/
/*****************************************************************************/

int main( int argc, char **argv )
{
  int i;
  Dynamics *d;
  Controller *c;
  double last_time = 0;
  double the_time;
  int status;
  double qdd_d[N_ACC];
  double forces_d[N_DS_F];

  last_time = get_time();

  d = create_dynamics();
  c = create_controller();
  init_data( d );

  d->xh = 0.0;
  d->yh = 0.99*(d->thigh_length + d->shank_length);
  d->ato = 0.0;
  d->xal = d->xh + 0.01;
  d->yal = 0.0;
  d->xar = d->xh - 0.01;
  d->yar = 0.0;

  d->xhd = 0.0;
  d->yhd = 0.0;
  d->wto = 0.0;
  d->xald = 0.0;
  d->yald = 0.0;
  d->xard = 0.0;
  d->yard = 0.0;

  d->time = 0.0;

  d->dynamics_state = D_DS;

  set_state( d );

  for( i = 0; i < N_U; i++ )
    {
      d->action[i] = 0;
    }

  // actions known to work for standing up
  d->action[U_AHL] = -1.0748;
  d->action[U_AKL] = 20.5409;
  d->action[U_AAL] = -5.6625;
  d->action[U_AHR] = 1.0747;
  d->action[U_AKR] = 20.8700;
  d->action[U_AAR] = -8.1412;

  accelerations( d, d->action, NULL ); // get initial accelerations
  forward_kinematics( d ); // capture foot forces

  init_controller( c, d );

  // 5000 is 5 seconds at 1kHz sampling rate
  // 5000 is 2.5 seconds at 2kHz sampling rate
  for ( i = 0; i < 5000; i++ )
    {
      run_controller( c, d );
      save_data( d, c );
      status = crash( d );
      print_crash_message( d, status );
      d->status = status;
      if ( status != NOT_CRASHED && status <= FATAL_CRASH )
	break;
      integrate_one_step( d, NULL, NULL );
    }

  write_the_mrdplot_file( d );

  the_time = get_time();
  printf( "%g seconds to simulate %g seconds.\n", the_time - last_time,
	  d->time );

  return 0;
}

/*****************************************************************************/
